<?php
/*
###################################################################################################
#
# mailform.php v1.0.3
# You must have either PHP4 or PHP5 running on your server to execute this script!
#
# Created:       17/01/2010
# Last Modified: 24/04/2010
#
# Copyright (c) 2010 Bergware International - All rights reserved.
#
# DESCRIPTION
# 
# mailform is an email processing script which works together with the Bergware ANTP template.
# The javascript part of the template creates a form with all necessary items.
# Do not use this PHP script in any other way, because it will surely fail!
#
# Before using this script set "$recipient" to your email address, this is a single address.
# This is the only mandatory setting to change, two more optional settings exist, see below.
# This setting acts as a protection mechanism against mail abuse from bots.
# Only messages with destination $recipient will be processed, otherwise an error occurs.
# Important: the address value set in the file "locale.xml" must correspond with $recipient.
#
###################################################################################################
MANDATORY
*/
$recipient = "";

/*
###################################################################################################
#
# By default the script will show the visitor's email address as the "From:" address.
# A minority of hosts insist that the "From:" address must be from the domain on the server.
# In this situation set your 'recipient' address in $sender, e.g. $sender="somebody@yourmail.com"
# Enabling this option means that the visitor's email address goes into a "Reply-To:" header.
# This means you can still hit "Reply" to respond to the visitor in the conventional way.
# The script relies on the field which contains "email" or "mail" to hold the visitor's address.
#
###################################################################################################
OPTIONAL
*/
$sender = "";

/*
###################################################################################################
#
# You may create a list of banned email addresses or IP addresses/ranges which can not send emails.
# Give email or IP addresses/ranges in lowercase without spaces between the quotes. For example:
# $banned = array("*@somedomain.com","user@domain.com","192.168.1.0/24","192.168.2.1");
#
###################################################################################################
OPTIONAL
*/
$banned = array();

/*
###################################################################################################
#
# The file "mailform_lng.php" contains the localized text for the error messages.
# At the moment this file contains English and Dutch translations, but you can add your own.
# For example: create 'french' and change the setting below to have your language settings loaded.
#
###################################################################################################
LOCALIZATION
*/
$language = "english";

/*
###################################################################################################
# END OF CONFIGURATION SETTINGS (no further modification required)
###################################################################################################
*/

@include ("mailform.lng.php");

define ("NL","\n");
define ("EMAIL","/e?mail/i");
define ("VALIDEMAIL","/^[_\.0-9a-z-]+@([0-9a-z][0-9a-z-]+\.)+[a-z]{2,3}$/i");
define ("USERNAME","/^[_\.0-9a-z-]+@/");

if (!defined("E_CLOSEBUTTON"))      define ("E_CLOSEBUTTON","Close");
if (!defined("E_ERRORHEADER"))      define ("E_ERRORHEADER","The following error occurred and stopped the mail processing");
if (!defined("E_BANNEDADDRESS"))    define ("E_BANNEDADDRESS","Your email address or IP address is banned");
if (!defined("E_INVALIDRECIPIENT")) define ("E_INVALIDRECIPIENT","Invalid recipient email address specified");
if (!defined("E_INVALIDSENDER"))    define ("E_INVALIDSENDER","Invalid sender email address specified");
if (!defined("E_MISSINGFIELD"))     define ("E_MISSINGFIELD","Missing mandatory entry:");
if (!defined("E_EMPTYFORM"))        define ("E_EMPTYFORM","You can not send an empty mail form");

function terminate($error){
 echo "<center><br><b>".E_ERRORHEADER."</b>";
 echo "<br><br><font color='red'>$error</font>";
 echo "<br><br><br><input type='button' value='".E_CLOSEBUTTON."' onClick='self.close()'>";
 exit;
}

function in_range($ipaddr,$iplist){
 $ip = ip2long($ipaddr);
 for ($i=0; $i<count($iplist); $i++){
  list($base,$bits) = explode("/",$iplist[$i]);
  if (!(($ip ^ ip2long($base)) & (0xffffffff << (32-$bits)))) return true;
 }
 return false;
}

function validate($address){
 global $banned;
 if (in_array($address,$banned) || in_array(preg_replace(USERNAME,"*@",$address),$banned) || in_range($_SERVER['REMOTE_ADDR'],$banned)){
  return false;
 }else{
  return true;
 }
}

function parser($input){
 global $banned,$subject,$recipient,$email,$href;
 foreach ($input as $key => $val){
  $keyIn = str_replace("_"," ",$key);
  $field = preg_replace("/\s?\*:?$/","",$keyIn);
  $value = trim($val);
  if ($value){
   $data = true;
   switch (strtolower($field)){
   case 'required':
    $required = explode(",",$value);
    break;
   case 'subject':
    $subject = utf8_decode($value);
    break;
   case 'recipient':
    if (!$recipient || $value != $recipient || !preg_match(VALIDEMAIL,$value)) terminate(E_INVALIDRECIPIENT);
    $receiver = true;
    break;
   case 'redirect':
    $href = $value;
    break;
   case (preg_match(EMAIL,$field)>0):
    if (!preg_match(VALIDEMAIL,$value)) terminate(E_INVALIDSENDER);
    if (!validate(strtolower($value))) terminate(E_BANNEDADDRESS);
    $email = $value;
   default:   
    if (is_numeric($field)){
     $content .= str_replace("\r\n",NL,$value).NL.NL;
    }else{
     $content .= "$field: $value".NL;
    }
   }
  }else{
   if (in_array($keyIn,$required)) $missing .= ($missing?"<br>":"").E_MISSINGFIELD." <b>$field</b>";
  }
 }
 if (!$data) terminate(E_EMPTYFORM);
 if (!$receiver) terminate(E_INVALIDRECIPIENT);
 if ($missing) terminate($missing);
 return utf8_decode($content);
}

$email = $recipient;
$message = parser($_POST);
$headers = "From: ".($sender?$sender."\r\nReply-To: $email":$email)."\r\nCc: $email\r\nX-Mailer: PHP/".phpversion();

mail($recipient,$subject,$message,$headers);
header("Location: ".(!$href?"mailform.html":$href));
?>